﻿InitEngine3D()
InitSprite()
InitKeyboard()


Structure Vector3
  x.f
  y.f
  z.f
EndStructure

Procedure Normalize(*V.Vector3)
  Define.f magSq, oneOverMag
 
  magSq = *V\x * *V\x + *V\y * *V\y + *V\z * *V\z
  If magsq > 0
    oneOverMag = 1.0 / Sqr(magSq)
    *V\x * oneOverMag
    *V\y * oneOverMag
    *V\z * oneOverMag
  EndIf
 
EndProcedure

Procedure CreateCylinder_(meshID, radius.f, height.f, numSegBase=16, numSegHeight=1, capped=1)
  Protected    Normal.Vector3
 
  CreateMesh(meshID)
 
  If (numSegBase < 1)
    numSegBase = 1
  EndIf
 
  If (numSegHeight < 1)
    numSegHeight = 1
  EndIf
 
  deltaAngle.f = #PI*2 / numSegBase
  deltaHeight.f = height / numSegHeight
  height2.f = height / 2.0
  offset = 0
 
  For i = 0 To numSegHeight
    For j = 0 To numSegBase
     
      x0.f = radius * Cos(j*deltaAngle)
      z0.f = radius * Sin(j*deltaAngle)
     
      Normal\x = x0
      Normal\y = 0
      Normal\z = z0
      Normalize(@Normal)
     
      MeshVertexPosition(x0, i*deltaHeight-height2, z0)
      MeshVertexNormal(Normal\x, Normal\y, Normal\z)
      MeshVertexTextureCoordinate(j / numSegBase, i / numSegHeight)
     
      If i <> numSegHeight
       
        MeshIndex(offset + numSegBase + 1)
        MeshIndex(offset)
        MeshIndex(offset + numSegBase)
        MeshIndex(offset + numSegBase + 1)
        MeshIndex(offset + 1)
        MeshIndex(offset)
      EndIf
      offset + 1
    Next
  Next   
  If capped
   
    ;low cap
    centerIndex = offset
    MeshVertexPosition(0, -height2, 0)
    MeshVertexNormal(0,-1,0)
    MeshVertexTextureCoordinate(1, 0)
   
    offset + 1
    For j = 0 To numSegBase
     
      x0.f = Cos(j*deltaAngle)
      z0.f = Sin(j*deltaAngle)
      x1.f = 0.1 * Cos(j*deltaAngle) * 0.1
      z1.f = 0.1 * Sin(j*deltaAngle) * 0.1
     
      MeshVertexPosition(radius*x0, -height2, radius*z0)
      MeshVertexNormal(0, -1, 0)
      MeshVertexTextureCoordinate(x1, z1)
     
      If j <> numSegBase
       
        MeshIndex(centerIndex)
        MeshIndex(offset)
        MeshIndex(offset + 1)
      EndIf
      offset + 1
    Next
   
    ; high cap
    centerIndex = offset;
   
    MeshVertexPosition(0, height2, 0)
    MeshVertexNormal(0, 1, 0)
    MeshVertexTextureCoordinate(1, 0)
   
    offset + 1
    For j = 0 To numSegBase
     
      x0.f = Cos(j*deltaAngle)
      z0.f = Sin(j*deltaAngle)
      x1.f = 0.1 * Cos(j*deltaAngle) * 0.1
      z1.f = 0.1 * Sin(j*deltaAngle) * 0.1
     
      MeshVertexPosition(x0 * radius, height2, radius * z0)
      MeshVertexNormal(0, 1, 0)
      MeshVertexTextureCoordinate(x1, z1)
     
      If j <> numSegBase
       
        MeshIndex(centerIndex)
        MeshIndex(offset + 1)
        MeshIndex(offset)
      EndIf
      offset + 1
    Next
  EndIf
 
  FinishMesh(#True)
EndProcedure


OpenWindow(0, 0, 0, 800, 600, "Cylinder", #PB_Window_ScreenCentered|#PB_Window_SystemMenu)
OpenWindowedScreen(WindowID(0), 0, 0, 1024, 768)
CreateCamera(0, 0, 0, 100, 100)
MoveCamera(0, 0, 0, 10)

Add3DArchive("mesh", #PB_3DArchive_FileSystem)

LoadMesh(0, "cyl.mesh")
CreateCylinder_(1, 0.16, 1.5, 30, 5, 1)

CreateTexture(0, 512, 512)
StartDrawing(TextureOutput(0))
Box(0, 0, 512, 512, $12567E)
Box(0, 0, 512, 128, $FFFFFF)
StopDrawing()
CreateMaterial(0, TextureID(0))
ScrollMaterial(0, 0.5, 0.4, #PB_Material_Animated)
CreateEntity(0,MeshID(0), MaterialID(0), -2, 0, 1)
CreateEntity(1, MeshID(1), MaterialID(0), 0, 0, 0)

Repeat
  
  Repeat
    ev = WindowEvent()
  Until ev = 0
  
  RotateEntity(0, 0.5, 0.2, 0.7, #PB_Relative)
  RotateEntity(1, 0.5, 0.2, -0.7, #PB_Relative)

  
  RenderWorld()
  
  ExamineKeyboard()
  
  FlipBuffers()
  
Until KeyboardPushed(#PB_Key_Escape)

; IDE Options = PureBasic 5.40 LTS Beta 9 (Windows - x64)
; CursorPosition = 147
; FirstLine = 133
; Folding = -
; EnableUnicode
; EnableXP
; CompileSourceDirectory